/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MESHITEM_H
#define MESHITEM_H
#include <QObject>
#include <QtCore>
#include <QtGui>
#include <QGLWidget>
// Forward Declarations
#include "QtOclMesh_global.h"


// Class Definitions
#include "point3d.h"
#include "size3d.h"
#include "meshengine.h"


/**********************************************************
**
**  MESH ITEM  (BASE CLASS)
**
**********************************************************/

class QTOCLMESHSHARED_EXPORT MeshItem : public QObject
{
  Q_OBJECT

  // MEMBERS

private:
  Point3D<float>          pos_;
  Size3D<float>           size_;
  Size3D<float>           minimumSize_;
  Size3D<float>           maximumSize_;
  Point3D<float>          up_;
  QMap<QString,QVariant>  properties_;
  QMap<QString,QString>   propertyText_;
  bool                    selected_;
  Mesh*                   mesh_;

protected:
  QString                 displayText_;
  QString                 whatsThisText_;
  QString                 toolTipText_;
  QString                 itemName_;
  QColor                  fillColor_;
  QColor                  borderColor_;



public:
    MeshItem();
    MeshItem(const MeshItem& other);
    ~MeshItem();

    MeshItem& operator =(const MeshItem& other);
    virtual operator QVariant() const;
    virtual MeshItem* clone() const;


    const Mesh* mesh() const;
    virtual void setMesh(Mesh* mesh);

    // GETTERS

    QVariant getPropertyValue(const QString& fullPropertyName)const;
    const QString getPropertyText(const QString& fullPropertyName) const;
    const QMap<QString,QVariant> getPropertyMap() const;
    const QMap<QString,QString> getPropertyTextMap() const;

    const QString getDisplayText() const;
    const QString getWhatsThis() const;
    const QString getToolTip() const;
    const QColor getFillColor() const;
    const QColor getBorderColor() const;

    const Point3D<float> pos() const;
    const Size3D<float> size() const;
    const Point3D<float>& up() const;
    const Size3D<float> minimumSize() const;
    const Size3D<float> maximumSize() const;
    const QString& itemName() const;


    // SETTERS

    void setPropertyValue(const QString& fullPropertyName,
                                  const QVariant value);
    void setPropertyTextMap(const QMap<QString,QString> newMap);
    void setPropertyMap(const QMap<QString,QVariant> newMap);
    virtual void setPos(const Point3D<float> pos);
    virtual void setSize(const Size3D<float> size);
    virtual void setUp(const Point3D<float> up);
    void setMinimumSize(const Size3D<float> minimumSize);
    void setMaximumSize(const Size3D<float> maximumSize);
    void setItemName(const QString& itemName);
    void setFillColor(const QColor& color);
    void setBorderColor(const QColor& color);

    void setSelected(bool selected);
    const bool isSelected() const;

    // MODIFIERS
    void adjustSize(const Size3D<float>& deltaSize);
    void adjustPos(const Point3D<float>& deltaPos);

    virtual void addToMesh(MeshEngine& meshEngine);
    virtual void draw();

    // This method is used by the GUI to generate images for widgets
    virtual const QPixmap
    generateImage(const QSize& size = QSize(255,255),
                  const QColor border = QColor(Qt::black),
                  const QColor fill = QColor(Qt::transparent));



protected:
    QString getFullPropertyName(const QString& propertyName) const;
    void addProperty(const QString& propertyName,
                     const QString& newPropertyText = QString(),
                     const QVariant defaultValue=QVariant(0.0f));
    void removeProperty(const QString& propertyName);

    const QVariant getPropertyValue_(const QString& propertyName) const;
    void setPropertyValue_(const QString& propertyName, const QVariant value);
    const QString getPropertyText_(const QString& propertyName) const;

    QPixmap getPixmap(const QSize& size)const;
    void setupPainter(QPainter& painter,
                      const QColor border,
                      const QColor fill);

    virtual void setupGL();

};

Q_DECLARE_METATYPE(MeshItem);

QTOCLMESHSHARED_EXPORT QDataStream&
operator<<(QDataStream &out, const MeshItem& item);

QTOCLMESHSHARED_EXPORT QDataStream&
operator>>(QDataStream &in, MeshItem& item);

QTOCLMESHSHARED_EXPORT QDebug
operator<<(QDebug dbg,const MeshItem& item);


QTOCLMESHSHARED_EXPORT Point3D<float>
getBottomBackLeft(const MeshItem* item);

QTOCLMESHSHARED_EXPORT Point3D<float>
getTopFrontRight(const MeshItem* item);

QTOCLMESHSHARED_EXPORT Point3D<float>
getBottomBackLeft(const QList<MeshItem*> items);

QTOCLMESHSHARED_EXPORT Point3D<float>
getTopFrontRight(const QList<MeshItem*> items);

QTOCLMESHSHARED_EXPORT Point3D<float>
getBoundingBoxSize(const QList<MeshItem*> items);

#endif // MESHITEM_H
