/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef OCLMANAGER_H
#define OCLMANAGER_H
#include "QtOclMesh_global.h"
#include <QObject>
#include <qclcontext.h>
#include <qclcontextgl.h>
#include "QtOCLMesh.h"

class QTOCLMESHSHARED_EXPORT OCLManager : public QObject
{
    Q_OBJECT
public:

private:
    bool              hostCompatible;
    QString           kernelFilePath;
    bool              profilingEnabled;

    QCLContextGL*     contextGL;
    QCLContext*       context;
    QCLProgram        program;
    QCLCommandQueue   commandQueue;
    
    QCLKernel         exciteKernel;
    QCLKernel         sampleKernel;
    QCLKernel         scatterKernel;
    QCLKernel         impulseInterchangeKernel;
    quint64           exciteKernelRunTime;
    quint64           sampleKernelRunTime;
    quint64           scatterKernelRunTime;
    quint64           interchangeKernelRunTime;


    QCLVector<float> boundaryTypeTable;
    QCLVector<float> voltages;
    QCLVector<unsigned short> boundaries;
    QCLVector<float> properties;
    QCLVector<unsigned int> probeInformation_;
    QCLVector<float> probeData_;
    QCLVector<unsigned int> sourceInformation_;
    QCLVector<float> sourceData_;

    size_t numSources_;
    size_t numProbes_;

public:
    explicit OCLManager(QString initKernelFilePath,QObject *parent = 0);
    OCLManager(const OCLManager &other);
    OCLManager& operator=(const OCLManager& other);
    ~OCLManager();

    const bool isHostCompatible() const;

    // MEMORY MANAGEMENT FUNCTIONS
    // HOST -> DEVICE
    void copyBoundaryTypeTableToDevice(float* data, size_t size);
    void copyBoundariesToDevice(unsigned short* data, size_t size);
    void copyPropertiesToDevice(float* data, size_t size);
    void copyProbesToDevice(int numProbes,
                            QVector<unsigned int>& probeInfoVector,
                            int probeBufferSize);

    void copySourcesToDevice(int numSources,
                             QVector<unsigned int>& sourceInfoVector,
                             QVector<float>& sourceDataVector);
    void createVoltageBuffer(float* data, size_t size);

    // DEVICE -> HOST
    void copyVoltagesToHost(float* data, size_t size);
    void copyProbesToHost(float* data, size_t size);

    // SIMULATION ROUTINES
    // May want to have the following functions take no arguments
    // and move the source and probe list arguments to different
    // methods.
    void exciteMesh(int currentIteration);
    void sampleMesh(int currentIteration);
    void scatterImpulses();
    void impulseInterchange();

    void releaseBuffers();

    void setProfilingEnabled(const bool enabled);
    const bool isProfilingEnabled() const;
    const quint64 totalSamplingKernelRunTime() const;
    const quint64 totalExcitationKernelRunTime() const;
    const quint64 totalScatteringKernelRunTime() const;
    const quint64 totalInterchangeKernelRunTime() const;


private:
    bool setupOpenCL();
    bool setupOpenCLContext();
    bool createCommandQueue();
    bool createProgramFromFile(const QString fileName);
    bool buildProgram();
    bool createKernels();
    QCLWorkSize getGlobalWorkSize(const MeshSize& meshSize,QCLWorkSize localSize);

signals:

public slots:
    void oclFinish();
    void wait(int mSecs);
    void setupKernels(const MeshSize meshSize, size_t workItemSharedMemSize = 0, const MeshSize localWorkSize=MeshSize());
    void setupScatteringKernel(const MeshSize meshSize, size_t workItemSharedMemSize = 0, const MeshSize localWorkSize=MeshSize());
    void setupImpulseInterchangeKernel(const MeshSize meshSize, size_t workItemSharedMemSize = 0, const MeshSize localWorkSize=MeshSize());
    void setupExcitationKernel(const MeshSize meshSize, size_t workItemSharedMemSize = 0, const MeshSize localWorkSize=MeshSize());
    void setupSamplingKernel(const MeshSize meshSize, size_t workItemSharedMemSize = 0, const MeshSize localWorkSize=MeshSize());



};

#endif // OCLMANAGER_H
