/*******************************************************************************
**  FILE:	    meshsize.h
**  DESCRIPTION:    This class stores the size information for the mesh class.
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MESHSIZE_H
#define MESHSIZE_H
#include <Qtcore>
#include "QtOclMesh_global.h"
#include <stdlib.h>
#include "point3d.h"

template <typename T>
class /*QTOCLMESHSHARED_EXPORT*/ Size3D
{

private:
    T width_;
    T height_;
    T depth_;

public:

    Size3D() :
            width_(0),
            height_(0),
            depth_(0)
    {
    }



    Size3D(T width,T height) :
            width_(width),
            height_(height),
            depth_(0)
    {
    }

    Size3D(T width, T height, T depth) :
            width_(width),
            height_(height),
            depth_(depth)
    {
    }



    Size3D(const Size3D<T> &other)
    {
        width_ = other.width_;
        height_ = other.height_;
        depth_ = other.depth_;
    }



    Size3D(const Point3D<T> &other)
    {
      width_   = other.cx();
      height_  = other.cy();
      depth_   = other.cz();
    }



    //~Size3D()
    //{
    //}



    Size3D<T>& operator=(const Size3D<T> &other)
    {
        if(&other!=this){
            width_ = other.width_;
            height_ = other.height_;
            depth_ = other.depth_;
        }
        return *this;
    }



    bool operator==(const Size3D<T> &other) const
    {
        return (width_==other.width_)&
                (height_==other.height_)&
                (depth_==other.depth_);
    }


    bool operator!=(const Size3D<T>& other) const
    {
      return !(*this == other);
    }



    Size3D<T>& operator+=(const Size3D<T> &other)
    {
      width_  += other.width_;
      height_ += other.height_;
      depth_  += other.depth_;
      return *this;
    }



    Size3D<T> operator+(const Size3D<T> &other)
    {
      Size3D<T> retSize = *this;
      retSize += other;
      return retSize;
    }



    Size3D<T>& operator-=(const Size3D<T> &other)
    {
      width_  -= other.width_;
      height_ -= other.height_;
      depth_  -= other.depth_;
      return *this;
    }


    Size3D&
    operator*=(const Size3D<T> &other)
    {
      width_  *= other.width_;
      height_ *= other.height_;
      depth_  *= other.depth_;
      return *this;
    }


    Size3D&
    operator*(const Size3D<T> &other) const
    {
      return Size3D<T>(*this) *= other;
    }


    Size3D&
    operator/=(const T value)
    {
      width_  /= value;
      height_ /= value;
      depth_  /= value;
      return *this;
    }


    Size3D&
    operator/(const T value) const
    {
      return Size3D<T>(*this) /= value;
    }


    Size3D&
    operator*=(const T value)
    {
      width_  *= value;
      height_ *= value;
      depth_  *= value;
      return *this;
    }



    Size3D&
    operator*(const T value) const
    {
      return Size3D<T>(*this) *= value;
    }



    // Setters

    inline void setWidth(const T width){
        width_ = width;
    }



    inline void setHeight(const T height){
        height_ = height;
    }



    inline void setDepth(const T depth){
        depth_ = depth;
    }

    // Getters
    inline const T width()const{
        return width_;
    }



    inline const T height()const{
        return height_;
    }



    inline const T depth()const{
        return depth_;
    }



    inline const T area()const{
        return qAbs(width_*height_);
    }



    inline const T volume()const{
        return qAbs(width_*height_*depth_);
    }


    Point3D<T> toPoint3D() const
    {
      return Point3D<T>(width_,height_,depth_);
    }

};


template <typename T> Size3D<T>
Min(const Size3D<T> &a, const Size3D<T> &b)
{
  Size3D<T> retSize;
  retSize.setWidth(qMin(a.width(),b.width()));
  retSize.setHeight(qMin(a.height(),b.height()));
  retSize.setDepth(qMin(a.depth(),b.depth()));
  return retSize;
}



template <typename T> Size3D<T>
Max(const Size3D<T> &a, const Size3D<T> &b)
{
  Size3D<T> retSize;
  retSize.setWidth(qMax(a.width(),b.width()));
  retSize.setHeight(qMax(a.height(),b.height()));
  retSize.setDepth(qMax(a.depth(),b.depth()));
  return retSize;
}


template <typename T> Size3D<T>
abs(const Size3D<T> &a)
{
  Size3D<T> retSize;
  retSize.setWidth(qAbs(a.width()));
  retSize.setHeight(qAbs(a.height()));
  retSize.setDepth(qAbs(a.depth()));
  return retSize;
}


template <typename T> QDebug operator<<(QDebug dbg,const Size3D<T>& size){
  dbg.nospace()<<"\nSize(w,h,d):("<<size.width()<<","<<size.height()<<","<<size.depth()<<")";
  return dbg.maybeSpace();
}



template <typename T> QDataStream& operator<<(QDataStream &out, const Size3D<T>& size){
  out << size.width()
      << size.height()
      << size.depth();
  return out;
}



template <typename T> QDataStream& operator>>(QDataStream &in, Size3D<T>& size){
  T tmp[3];
  in  >> tmp[0]
      >> tmp[1]
      >> tmp[2];

  size.setWidth( tmp[0] );
  size.setHeight( tmp[1] );
  size.setDepth( tmp[2] );

  return in;
}


// TYPE DEFINITIONS

typedef Size3D<int> MeshSize;


#endif // Size3D_H
