/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/
#ifndef TLMENGINE2DGPU_H
#define TLMENGINE2DGPU_H
#include "tlmengine2d.h"
#include "QtOclMesh_global.h"

class OCLManager;


class QTOCLMESHSHARED_EXPORT TLMEngine2DGPU : public TLMEngine2D
{
    Q_OBJECT

private:
    OCLManager* oclManager_;
    int         probeBufferSize_;
    float *     hostProbeBuffer_;
    int         lastProbeUpdateIteration_;

public:

    TLMEngine2DGPU(const TLMEngine2DGPU& other);
    TLMEngine2DGPU(Mesh* initMesh);
    ~TLMEngine2DGPU();

    TLMEngine2DGPU& operator=(const TLMEngine2DGPU& other);
    virtual MeshEngine* clone() const;

    virtual void createSimulation();
    virtual void simulateIteration( int iteration );
    virtual void updateHost();



protected:
    virtual void allocateResources(const MeshSize &meshSize);
//    virtual void initializeResources(const MeshSize &meshSize);
    virtual void releaseResources();

    virtual void scatterImpulses();
    virtual void impulseInterchange();
    virtual void exciteMesh(int iteration);
    virtual void sampleMesh(int iteration);

private:
    void updateHostProbes(int iteration);
    void copySourcesToDevice();
    void copyProbesToDevice();
    QVector<unsigned int> packSourceInformation(SourceMeshItem* source) const;
    QVector<unsigned int> packProbeInformation(ProbeMeshItem* source) const;

};

#endif // TLMENGINE2DGPU_H
