/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "inc/materialmeshitems.h"


/**********************************************************
**
**  MATERIAL MESH ITEM
**
**********************************************************/


//-----------------------------------------------------------------------------
// CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


MaterialMeshItem::MaterialMeshItem():
	MeshItem()
{
    displayText_    = QString("Material");
    whatsThisText_  = QString("Material mesh items for"
                              " representing electromagnetic materials.");
    toolTipText_    = QString("Material Items");
    fillColor_      = QColor(Qt::lightGray);
    borderColor_    = QColor(Qt::transparent);

    addProperty(QString("relative permittivity"),QString("Er"),QVariant(5.0f));
    addProperty(QString("conductance"),QString("sigma"),QVariant(0.0f));
}



MaterialMeshItem::MaterialMeshItem(const MaterialMeshItem &other):
	MeshItem(other)
{
}



MaterialMeshItem::~MaterialMeshItem()
{
}


//-----------------------------------------------------------------------------
// OPERATOR OVERLOADS
//-----------------------------------------------------------------------------


MaterialMeshItem&
MaterialMeshItem::operator=(const MaterialMeshItem& other)
{
  if(&other!=this){
    MeshItem::operator =(other);
  }
  return *this;
}



MeshItem*
MaterialMeshItem::clone() const
{
  return new MaterialMeshItem(*this);
}



MaterialMeshItem::operator QVariant() const
{
  return QVariant::fromValue(*this);
}



//-----------------------------------------------------------------------------
// PROTECTED METHODS
//-----------------------------------------------------------------------------


void
MaterialMeshItem::setNodeProperty(MeshEngine& meshEngine,
                                  MeshPoint pos,
                                  int propertyNum,
                                  float value)
{
  //meshEngine.setNodeProperty(pos,propertyNum,value);
}



void
MaterialMeshItem::setEpsilonR(MeshEngine &meshEngine,
                                    const MeshPoint &pos)
{
  float epsilonR = getPropertyValue(QString("relative permittivity")).toFloat();
  //meshEngine.setNodeProperty(pos,1,4.0*(epsilonR-1));

}



void
MaterialMeshItem::setConductance(MeshEngine &meshEngine,
                                       const MeshPoint &pos)
{
  float conductance = getPropertyValue(QString("conductance")).toFloat();
  //meshEngine.setNodeProperty(pos,2,conductance);
}




//-----------------------------------------------------------------------------
// STREAM OPERATORS
//-----------------------------------------------------------------------------


QDebug operator<<(QDebug dbg,const MaterialMeshItem& item){
  dbg.nospace()<<"\nMaterial Mesh Item:"<< (MeshItem)item;
  return dbg.maybeSpace();
}



QDataStream& operator<<(QDataStream &out, const MaterialMeshItem& item){
  out << (MeshItem)item;
  return out;
}



QDataStream& operator>>(QDataStream &in, MaterialMeshItem& item){
  MeshItem tmpItem;
  in>>tmpItem;
  (MeshItem)item = tmpItem;
  return in;
}





/**********************************************************
**
**  RECTANGLE DIELECTRIC MATERIAL MESH ITEM
**
**********************************************************/


//-----------------------------------------------------------------------------
//  CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


RectangularMaterial::RectangularMaterial():
	MaterialMeshItem()
{
    displayText_    = QString("Rectangle");
    whatsThisText_  = QString("Mesh Item representing a "
                              "rectangular dielectric material.");
    toolTipText_    = QString("Rectangular Dielectric Material Item");
}



RectangularMaterial::RectangularMaterial(
    const RectangularMaterial &other):
    MaterialMeshItem(other)
{
}



RectangularMaterial::~RectangularMaterial()
{
}



//-----------------------------------------------------------------------------
//  OPERATOR OVERLOADS
//-----------------------------------------------------------------------------


RectangularMaterial& RectangularMaterial::operator=(
    const RectangularMaterial &other)
{
  if(&other!=this){
    MaterialMeshItem::operator =(other);
  }
  return *this;
}



MeshItem*
RectangularMaterial::clone() const
{
  return new RectangularMaterial(*this);
}



RectangularMaterial::operator QVariant() const
{
  return QVariant::fromValue(*this);
}



//-----------------------------------------------------------------------------
//  VIRTUAL METHOD DEFINITIONS
//-----------------------------------------------------------------------------


void
RectangularMaterial::addToMesh(MeshEngine &meshEngine)
{
//  QRect rect = getMeshRect();

//  MeshPoint point;
//  for(point.ry()=rect.top();point.ry()<=rect.bottom();++point.ry()){
//    for(point.rx()=rect.left();point.rx()<=rect.right();++point.rx()){
//	    setEpsilonR(mesh,point);
//	    setConductance(mesh,point);
//    }
//  }
}



void
RectangularMaterial::draw()
{

}



const QPixmap
RectangularMaterial::generateImage(const QSize &size,
                                           const QColor border,
                                           const QColor fill)
{
  QPixmap pixmap = getPixmap(size);
  QPainter painter(&pixmap);
  setupPainter(painter,border,fill);

  QRectF displayRect(QPointF(0,0),size);
  painter.fillRect(displayRect,painter.brush());
  painter.drawRect(displayRect);

  return pixmap;
}



//-----------------------------------------------------------------------------
// STREAM OPERATORS
//-----------------------------------------------------------------------------



QDataStream&
operator<<(QDataStream &out, const RectangularMaterial& item)
{
  out << (MaterialMeshItem)item;
  return out;
}



QDataStream&
operator>>(QDataStream &in, RectangularMaterial& item)
{
  MaterialMeshItem tmpItem;
  in >> tmpItem;
  (MaterialMeshItem)item = tmpItem;
  return in;
}



QDebug
operator<<(QDebug dbg,const RectangularMaterial& item)
{
  dbg.nospace() << "\nRectangular Dielectric Material Mesh Item:"
      << (MaterialMeshItem)item;
  return dbg.maybeSpace();
}
