/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "inc/QtOclMesh_global.h"
#include "inc/meshitem.h"
#include "inc/point3d.h"
#include "inc/size3d.h"
#include "inc/meshengine.h"


//-----------------------------------------------------------------------------
// CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


MeshItem::MeshItem()
  : QObject         ( 0 )
  , properties_     ( QMap<QString,QVariant>() )
  , propertyText_   ( QMap<QString,QString>() )
  , selected_       ( false )
  , displayText_    ( QString() )
  , whatsThisText_  ( QString() )
  , toolTipText_    ( QString() )
  , fillColor_      ( QColor(Qt::transparent) )
  , borderColor_    ( QColor(Qt::black) )
  , mesh_           ( 0 )
{
}



MeshItem::MeshItem(const MeshItem &other)
  : QObject         ( 0 )
  , pos_            ( other.pos_ )
  , size_           ( other.size_ )
  , up_             ( other.up_ )
  , properties_     ( other.properties_ )
  , propertyText_   ( other.propertyText_ )
  , selected_       ( false )
  , displayText_    ( other.displayText_ )
  , whatsThisText_  ( other.whatsThisText_ )
  , toolTipText_    ( other.toolTipText_ )
  , fillColor_      ( other.fillColor_ )
  , borderColor_    ( other.borderColor_ )
  , mesh_           ( other.mesh_ )
{
}



MeshItem::~MeshItem()
{
}



//-----------------------------------------------------------------------------
// OPERATOR OVERLOADS
//-----------------------------------------------------------------------------


MeshItem&
MeshItem::operator =(const MeshItem &other)
{
  if(this!=&other){
    pos_            = other.pos_;
    size_           = other.size_;
    up_             = other.up_;
    displayText_    = other.displayText_;
    whatsThisText_  = other.whatsThisText_;
    toolTipText_    = other.toolTipText_;
    fillColor_      = other.fillColor_;
    borderColor_    = other.borderColor_;
    properties_     = other.properties_;
    propertyText_   = other.propertyText_;
    mesh_           = other.mesh_;
  }  
  return *this;
}



MeshItem::operator QVariant() const
{
  return QVariant::fromValue(*this);
}



MeshItem*
MeshItem::clone() const
{
  return new MeshItem(*this);
}




//-----------------------------------------------------------------------------
// PUBLIC METHODS
//-----------------------------------------------------------------------------


const Mesh*
MeshItem::mesh() const
{
  return mesh_;
}


void
MeshItem::setMesh(Mesh *mesh)
{
  mesh_ = mesh;
  setParent((QObject*)mesh_);
}




// GETTERS



QVariant
MeshItem::getPropertyValue(const QString &fullPropertyName) const
{
  if( properties_.contains( fullPropertyName ) ){
    return properties_[ fullPropertyName ];
  }
  else{
    return QVariant(0);
  }
}



const QString
MeshItem::getPropertyText(const QString &fullPropertyName) const
{
  if( properties_.contains( fullPropertyName ) ){
    return propertyText_[ fullPropertyName ];
  }
  else{
    return QString();
  }
}



const QMap<QString,QVariant>
MeshItem::getPropertyMap() const
{
  return properties_;
}



const QMap<QString,QString>
MeshItem::getPropertyTextMap() const
{
  return propertyText_;
}



const QString
MeshItem::getDisplayText() const
{
  return displayText_;
}



const QString
MeshItem::getWhatsThis() const
{
  return whatsThisText_;
}



const QString
MeshItem::getToolTip() const
{
  return toolTipText_;
}



const QColor
MeshItem::getFillColor() const
{
  QColor retColor;
  if(isSelected()){
    retColor = Qt::lightGray;
  }else{
    retColor = fillColor_;
  }

  return retColor;
}



const QColor
MeshItem::getBorderColor() const
{
  QColor retColor;
  if(isSelected()){
    retColor = Qt::gray;
  }else{
    retColor = borderColor_;
  }
  return retColor;
}



const Point3D<float>
MeshItem::pos() const
{
  return pos_;
}



const Size3D<float>
MeshItem::size() const
{
  return size_;
}



const Point3D<float>&
MeshItem::up() const
{
  return up_;
}



const Size3D<float>
MeshItem::minimumSize() const
{
  return minimumSize_;
}



const Size3D<float>
MeshItem::maximumSize() const
{
  return maximumSize_;
}


const QString&
MeshItem::itemName() const
{
  return itemName_;
}


// SETTERS


void
MeshItem::setPropertyValue(const QString &fullPropertyName, const QVariant value)
{
  if(properties_.contains(fullPropertyName)){
    properties_[ fullPropertyName ] = value;
  }
}



void
MeshItem::setPropertyTextMap(const QMap<QString,QString> newMap)
{
  propertyText_ = newMap;
}



void
MeshItem::setPropertyMap(const QMap<QString,QVariant> newMap)
{
  properties_ = newMap;
}



void
MeshItem::setPos(const Point3D<float> pos)
{
  pos_ = pos;
}



void
MeshItem::setSize(const Size3D<float> size)
{
  size_ = size;
}



void
MeshItem::setUp(const Point3D<float> up)
{
  up_ = up;
}



void
MeshItem::setMinimumSize(const Size3D<float> minimumSize)
{
  minimumSize_ = minimumSize;
}



void
MeshItem::setMaximumSize(const Size3D<float> maximumSize)
{
  maximumSize_ = maximumSize;
}



void
MeshItem::setItemName(const QString &itemName)
{
  itemName_ = itemName;
}



void
MeshItem::setFillColor(const QColor &color)
{
  fillColor_ = color;
}



void
MeshItem::setBorderColor(const QColor &color)
{
  borderColor_ = color;
}



void
MeshItem::setSelected(bool selected)
{
  selected_ = selected;
}



const bool
MeshItem::isSelected() const
{
  return selected_;
}



// MODIFIERS


void
MeshItem::adjustSize(const Size3D<float> &deltaSize)
{
  setSize(size_ + deltaSize);
}



void
MeshItem::adjustPos(const Point3D<float> &deltaPos)
{
  setPos(pos_+ deltaPos);
}



void
MeshItem::addToMesh(MeshEngine &meshEngine)
{

}


void
MeshItem::draw()
{

}


const QPixmap
MeshItem::generateImage(const QSize &size, const QColor border, const QColor fill)
{
  QPixmap pixmap = getPixmap(size);
  QPainter painter(&pixmap);
  setupPainter(painter,border,fill);

  painter.setBrush(QBrush(fill,Qt::BDiagPattern));

  QRectF displayRect(QPointF(0,0),size);
  painter.fillRect(displayRect,painter.brush());
  painter.drawRect(displayRect);
  painter.drawText(displayRect,QString("???"));
  painter.end();
  return pixmap;
}




//-----------------------------------------------------------------------------
// PROTECTED METHODS
//-----------------------------------------------------------------------------


QString
MeshItem::getFullPropertyName(const QString &propertyName) const
{
  QString fullPropertyName = QString(this->metaObject()->className());
  fullPropertyName.append("/").append(propertyName);
  return fullPropertyName;
}


void
MeshItem::addProperty(const QString &propertyName,
                      const QString &newPropertyText,
                      const QVariant defaultValue)
{
  QString fullPropertyName = getFullPropertyName(propertyName);
  if( properties_.contains( fullPropertyName ) ){
    properties_[fullPropertyName] = defaultValue;
  }
  else{
    properties_.insert(fullPropertyName,defaultValue);
  }

  if( propertyText_.contains( fullPropertyName ) ){
    propertyText_[fullPropertyName] = newPropertyText;
  }
  else{
    propertyText_.insert(fullPropertyName,newPropertyText);
  }
}



void
MeshItem::removeProperty(const QString &propertyName)
{
  QString fullPropertyName = getFullPropertyName(propertyName);
  if( properties_.contains( fullPropertyName ) ){
    properties_.remove(fullPropertyName);
    propertyText_.remove(fullPropertyName);
  }
}


const QVariant
MeshItem::getPropertyValue_(const QString &propertyName) const
{
  QString fullPropertyName = getFullPropertyName(propertyName);
  return getPropertyValue(fullPropertyName);
}


void
MeshItem::setPropertyValue_(const QString &propertyName, const QVariant value)
{
  QString fullPropertyName = getFullPropertyName(propertyName);
  setPropertyValue(fullPropertyName,value);
}


const QString
MeshItem::getPropertyText_(const QString &propertyName) const
{
  QString fullPropertyName = getFullPropertyName(propertyName);
  return getPropertyText(fullPropertyName);
}








QPixmap
MeshItem::getPixmap(const QSize &size) const
{
  QPixmap pixmap(size);
  pixmap.fill(Qt::transparent);
  return pixmap;
}



void
MeshItem::setupPainter(QPainter& painter,
                       const QColor border,
                       const QColor fill)
{
  painter.setBrush(QBrush(fill,Qt::SolidPattern));
  painter.setPen(QPen(border, 4));
}



void
MeshItem::setupGL()
{

}


//-----------------------------------------------------------------------------
// STREAM OPERATORS
//-----------------------------------------------------------------------------


QDebug
operator<<(QDebug dbg,const MeshItem& item){
  dbg.nospace()<<"\nDisplay Text:"<<item.getDisplayText();
  dbg.nospace()<<"\nTool Tip:"<<item.getToolTip();
  dbg.nospace()<<"\nWhats This:"<<item.getWhatsThis();
  dbg.nospace()<<"\nPos: "<< item.pos();
  dbg.nospace()<<"\nSize: "<< item.size();

  QMap<QString,QVariant>::ConstIterator i;
  for(i=item.getPropertyMap().constBegin();i!=item.getPropertyMap().constEnd();++i){
    dbg.nospace()<<"\nProperty "<<item.getPropertyText(i.key())<<" = "<<i.value();
  }
  return dbg.maybeSpace();
}



QDataStream&
operator<<(QDataStream &out, const MeshItem& item){

  out << item.pos()
      << item.size()
      << item.up()
      << item.getPropertyMap()
      << item.getPropertyTextMap()
      << item.itemName()
      << item.getFillColor()
      << item.getBorderColor()
      << item.minimumSize()
      << item.maximumSize();


  return out;
}



QDataStream&
operator>>(QDataStream &in, MeshItem& item){
  Point3D<float> tmpPos,tmpUp;
  Size3D<float> tmpSize,tmpMinSize,tmpMaxSize;
  QMap<QString,QVariant> newProps;
  QMap<QString,QString> newPropText;
  QString tmpItemName;
  QColor tmpFillColor, tmpBorderColor;


  in  >> tmpPos
      >> tmpSize
      >> tmpUp
      >> newProps
      >> newPropText
      >> tmpItemName
      >> tmpFillColor
      >> tmpBorderColor
      >> tmpMinSize
      >> tmpMaxSize;

  item.setSize(tmpSize);
  item.setUp(tmpUp);
  item.setPropertyMap(newProps);
  item.setPropertyTextMap(newPropText);
  item.setItemName(tmpItemName);
  item.setFillColor(tmpFillColor);
  item.setBorderColor(tmpBorderColor);
  item.setMinimumSize(tmpMinSize);
  item.setMaximumSize(tmpMaxSize);
  item.setPos(tmpPos);


  return in;
}



Point3D<float>
getBottomBackLeft(const MeshItem * item)
{
  Point3D<float> retPoint = item->pos();
  return Min(retPoint,retPoint+item->size().toPoint3D());
}



Point3D<float>
getTopFrontRight(const MeshItem * item)
{
  Point3D<float> retPoint = item->pos();
  return Max(retPoint,retPoint+item->size().toPoint3D());
}



Point3D<float>
getBottomBackLeft(const QList<MeshItem *> items)
{
  Point3D<float> retPoint = items.first()->pos();
  foreach(MeshItem* item, items){
    retPoint = Min(retPoint,getBottomBackLeft(item));
  }
  return retPoint;
}



Point3D<float>
getTopFrontRight(const QList<MeshItem *> items)
{
  Point3D<float> retPoint = items.first()->pos();
  foreach(MeshItem* item, items){
    retPoint = Max(retPoint,getTopFrontRight(item));
  }
  return retPoint;
}



Size3D<float>
boundingBoxSize(const QList<MeshItem *> items)
{
  Size3D<float> retSize = Size3D<float>(getTopFrontRight(items)-
                                        getBottomBackLeft(items));
  return retSize;
}
