/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "inc/probemeshitems.h"
#include "inc/mesh.h"



/**********************************************************
**
**  PROBE MESH ITEM
**
**********************************************************/



//-----------------------------------------------------------------------------
// CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


ProbeMeshItem::ProbeMeshItem():
    MeshItem()
{
  displayText_    = QString("Probe");
  whatsThisText_  = QString("Probe items are used for sampling the mesh "
                            "structure during simulation.");
  toolTipText_    = QString("Probe Items");
  fillColor_      = QColor(255,0,0,128);
  borderColor_    = QColor(255,0,0,200);
}



ProbeMeshItem::ProbeMeshItem(const ProbeMeshItem &other):
    MeshItem  ( other ),
    data_     ( other.data_ )
{
}



ProbeMeshItem::~ProbeMeshItem()
{
}



//-----------------------------------------------------------------------------
// OPERATOR OVERLOADS
//-----------------------------------------------------------------------------


const QPixmap
ProbeMeshItem::generateImage(const QSize &size,
                             const QColor border,
                             const QColor fill)
{
  QPixmap pixmap = getPixmap(size);
  QPainter painter(&pixmap);
  setupPainter(painter,border,fill);

  painter.drawEllipse(QPoint(size.width()/2,size.height()/2),
                      size.width()/2,size.height()/2);
  painter.end();
  return pixmap;
}



ProbeMeshItem&
ProbeMeshItem::operator=(const ProbeMeshItem& other)
{
  if(&other!=this){
    data_ = other.data_;
    MeshItem::operator =(other);
  }
  return *this;
}



MeshItem*
ProbeMeshItem::clone() const
{
    return new ProbeMeshItem(*this);
}



ProbeMeshItem::operator QVariant() const
{
    return QVariant::fromValue(*this);
}



//-----------------------------------------------------------------------------
// PUBLIC METHODS
//-----------------------------------------------------------------------------


void ProbeMeshItem::draw()
{
  QColor fill = getFillColor();
  GLUquadric * sphereQuad;
  GLdouble radius = size().width()/2.0;
  GLint slices, stacks;
  slices = 10;
  stacks = 10;

  Point3D<float> position = pos();

  sphereQuad = gluNewQuadric();

  glPushMatrix();

  glTranslatef(position.cx(),position.cy(),position.cz());
  //glTranslatef(-radius,-radius,radius);

  glColor3f(fill.redF(),fill.greenF(),fill.blueF());
  gluSphere(sphereQuad,radius,slices,stacks);


  glPopMatrix();

}



void
ProbeMeshItem::addToMesh(MeshEngine &meshEngine)
{
  clearData();
  meshEngine.addProbe(this);
}



void ProbeMeshItem::setSize(const Size3D<float> size)
{
  // Do nothing
}



void
ProbeMeshItem::setMesh(Mesh *mesh)
{
  MeshItem::setMesh(mesh);
  if(this->mesh() != 0 ){
    QList<ProbeMeshItem*> probeList = mesh->filteredItems<ProbeMeshItem>();
    setItemName(tr("Probe %1").arg(probeList.count()));
    MeshItem::setSize(this->mesh()->getMeshResolution());
  }
}



void
ProbeMeshItem::clearData()
{
  data_.clear();
}



const QVector<float>&
ProbeMeshItem::data() const
{
  return data_;
}



QVector<float>&
ProbeMeshItem::data()
{
  return data_;
}



void
ProbeMeshItem::setData(const QVector<float>& vector)
{
  data_ = vector;
}



void
ProbeMeshItem::appendData(float data)
{
  data_.append(data);
}



void
ProbeMeshItem::appendData(float *newData, int appendedElements)
{
  for(int i = 0; i < appendedElements; ++i){
    appendData(newData[i]);
  }
}



//-----------------------------------------------------------------------------
// STREAM OPERATORS
//-----------------------------------------------------------------------------


QDebug operator<<(QDebug dbg,const ProbeMeshItem& item){
  dbg.nospace()<<"\nProbe Mesh Item:"<< (MeshItem)item;
  dbg.nospace()<<"Probe Data:"<<item.data();
  return dbg.maybeSpace();
}



QDataStream& operator<<(QDataStream &out, const ProbeMeshItem& item){
  out << (MeshItem)item
      << item.data();
  return out;
}



QDataStream& operator>>(QDataStream &in, ProbeMeshItem& item){
  MeshItem tmpItem;
  QVector<float> tmpVector;

  in  >> tmpItem
      >> tmpVector;

  (MeshItem)item = tmpItem;
  item.setData(tmpVector);
  return in;
}
