/*******************************************************************************
**  FILE:           
**  DESCRIPTION:    
**
**
**
**  AUTHOR:         Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:     Dr. Poman So
**
*******************************************************************************/

#include "dataset.h"
#include <QtGui>



//-----------------------------------------------------------------------------
// CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


DataSet::DataSet()
  : QObject(0)
  , data_ (&dataContainer_)
{
}



DataSet::DataSet(QVector<float> &data)
  : QObject(0)
  , data_ (&data)
{
  analyzeData();
}



DataSet::DataSet(const DataSet &other)
{
  if(other.data_ == &(other.dataContainer_)){
    dataContainer_ = other.dataContainer_;
    data_ = &dataContainer_;
  }else{
    data_ = other.data_;
  }
  range_      = other.range_;
  domain_     = other.domain_;
  title_      = other.title_;
  xAxisTitle_ = other.xAxisTitle_;
  yAxisTitle_ = other.yAxisTitle_;

}



//-----------------------------------------------------------------------------
// PUBLIC METHODS
//-----------------------------------------------------------------------------


DataSet&
DataSet::operator=(const DataSet& other)
{
  if(&other != this){
    if(other.data_ == &(other.dataContainer_)){
      dataContainer_ = other.dataContainer_;
      data_ = &dataContainer_;
    }else{
      data_ = other.data_;
    }
    range_      = other.range_;
    domain_     = other.domain_;
    title_      = other.title_;
    xAxisTitle_ = other.xAxisTitle_;
    yAxisTitle_ = other.yAxisTitle_;
  }
  return *this;
}



//-- SETTERS --\\


void
DataSet::setData()
{
  data_ = &dataContainer_;
  data_->clear();
  analyzeData();
}



void
DataSet::setData(QVector<float>& data)
{
  data_ = &data;
  if(data_ != 0){
    if(dataContainer_.count() > 0){
      dataContainer_.clear();
    }
    analyzeData();
  }
}



void
DataSet::setRange(const QVector2D range)
{
  range_ = range;
  emit rangeChanged();
}



void
DataSet::setDomain(const QVector2D domain)
{
  domain_ = domain;
  emit domainChanged();
}



void
DataSet::setTitle(const QString title)
{
  title_ = title;
  emit titleChanged(title_);
}



void
DataSet::setXAxisTitle(const QString title)
{
  xAxisTitle_ = title;
  emit xAxisTitleChanged(xAxisTitle_);
}



void
DataSet::setYAxisTitle(const QString title)
{
  yAxisTitle_ = title;
  emit yAxisTitleChanged(yAxisTitle_);
}



//-- GETTERS --\\


QVector<float>&
DataSet::data()
{
  return *data_;
}



const QVector<float>&
DataSet::data() const
{
  return *data_;
}



const QVector2D&
DataSet::range() const
{
  return range_;
}



const QVector2D&
DataSet::domain() const
{
  return domain_;
}



const QString&
DataSet::title() const
{
  return title_;
}



const QString&
DataSet::xAxisTitle() const
{
  return xAxisTitle_;
}



const QString&
DataSet::yAxisTitle() const
{
  return yAxisTitle_;
}




//-----------------------------------------------------------------------------
// PRIVATE METHODS
//-----------------------------------------------------------------------------


void
DataSet::analyzeData()
{
  if(data_->count() > 0){
    setDomain(QVector2D(0,data_->count()-1));
    float min = data_->first();
    float max = min;
    foreach(float dataPoint, *data_){
      max = qMax(max,dataPoint);
      min = qMin(min,dataPoint);
    }
    setRange(QVector2D(min,max));
  }else{
    setDomain(QVector2D(0,0));
    setRange(QVector2D(0,0));
  }
}

