/*******************************************************************************
**  FILE:           
**  DESCRIPTION:    
**
**
**
**  AUTHOR:         Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:     Dr. Poman So
**
*******************************************************************************/

#ifndef DATATRANSFORM_H
#define DATATRANSFORM_H

#include <QObject>
#include <QtGui>
#include "size3d.h"

class DataSet;
class DataViewer;

class DataTransform : public QObject
{
  Q_OBJECT
private:
  QMap<QString,QVariant>  options_;
  QMap<QString,QString>   optionText_;
  QFutureWatcher<void>    transformWatcher;

protected:
  DataViewer*             dataViewer_;
  QString                 displayText_;
  QString                 whatsThisText_;
  QString                 toolTipText_;
  Size3D<float>           meshResolution_;


public:
  explicit DataTransform(DataViewer* dataViewer);
  ~DataTransform();
  void transformData(const DataSet& srcData, DataSet& destData);

  QVariant getOptionValue(const QString& optionName)const;
  void setOptionValue(const QString& optionName,const QVariant& value);
  const QString getOptionText(const QString& optionName) const;
  const QMap<QString,QVariant>& getOptionMap() const;
  const QMap<QString,QString>& getOptionTextMap() const;

  const QString& getDisplayText() const;
  const QString& getWhatsThis() const;
  const QString& getToolTip() const;

  bool hasOptions() const;


protected:
  void addOption(const QString& optionName,
                   const QString& optionTitle = QString(),
                   const QVariant optionValue = QVariant(0.0f));
  void removeOption(const QString& optionName);
  virtual void computeNewData(const DataSet* srcData, DataSet* destData);

signals:
  void dataTransformed();

private slots:
  void transformationFinished();

};


class DataTransformDialog : public QDialog
{
  Q_OBJECT
private:
  DataTransform*          parent_;
  QFormLayout*            mainLayout_;
  QMap<QString,QVariant>  options_;
  QMap<QString,QString>   optionText_;
  QDialogButtonBox*       buttonBox;
  QMap<QString,QWidget*>  inputList_;

public:
  explicit DataTransformDialog(DataTransform* parent);
  const QMap<QString,QVariant>& readOptions() const;

protected:
  void accept();
  void reject();

private:
  void createDialog();

};

#endif // DATATRANSFORM_H
