/*******************************************************************************
**  FILE:           
**  DESCRIPTION:    
**
**
**
**  AUTHOR:         Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:     Dr. Poman So
**
*******************************************************************************/

#include <QObject>
#include "datatransforms.h"
#include "dataset.h"
#include "datatransform.h"


/******************************
**
**  DISCRETE FOURIER TRANSFORM
**
******************************/

DFTransform::DFTransform(DataViewer *dataViewer)
  : DataTransform(dataViewer)
{
  displayText_ = tr("F(w)");
  toolTipText_ = tr("Discrete Fourier Transform");
  whatsThisText_ = tr("Gives an approximate frequency domain "
                      "representation of the time domain signal.");

  addOption(QString("numFreqPoints"),tr("Number of Frequency Points"),QVariant(1000));
  addOption(QString("lowerFrequency"),tr("Lower Frequency (GHz)"),QVariant(0.0f));
  addOption(QString("upperFrequency"),tr("Upper Frequency (GHz)"),QVariant(25.0f));
}


DFTransform::~DFTransform()
{

}


void
DFTransform::computeNewData(const DataSet* srcData, DataSet* destData)
{
      const float TWOPI         = 6.283184;
      const float lightSpeed    = 299792458;

      int numFreqPoints   = getOptionValue("numFreqPoints").toInt();

      float f1            = getOptionValue("lowerFrequency").toFloat()*1E9;
      float f2            = getOptionValue("upperFrequency").toFloat()*1E9;

      float deltaL        = meshResolution_.width();

      float timeStep = deltaL/(lightSpeed*1.5);

      float d1  = f1*timeStep;
      float d2  = f2*timeStep;

      float frequencyRes  = (d2-d1)/numFreqPoints;

      float realF,imagF,trigTerm;

      destData->setData();
      const QVector<float>* data_ = &(srcData->data());

      float freq = d1;
      while(freq <= d2){
        realF = 0.0;
        imagF = 0.0;
        for(int i = 0; i < data_->count(); ++i){
          trigTerm = i*TWOPI*freq;
          realF += data_->at(i)*qCos(trigTerm);
          imagF += data_->at(i)*qSin(trigTerm);
        }
        destData->data().append(qSqrt(qPow(realF,2)+qPow(imagF,2)));
        freq += frequencyRes;

      }
      destData->analyzeData();
      destData->setTitle(tr("Frequency Domain"));
      destData->setXAxisTitle(tr("Frequency (GHz)"));
      destData->setYAxisTitle(tr("Amplitude"));
      destData->setDomain(QVector2D(f1/1E9,f2/1E9));
}
