/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "probeviewer.h"
#include "meshitems.h"
#include "mesh.h"

ProbeViewer::ProbeViewer(Mesh* mesh, QWidget *parent)
  : DataViewer(mesh,parent)
  , probe_ (0)
{
  createProbeCombo();
  connect(mesh_,SIGNAL(meshItemsModified(Mesh*)),
          this,SLOT(updateProbeCombo()));

  viewerToolLayout_->addWidget(probeCombo_);
}




//-----------------------------------------------------------------------------
// PRIVATE METHODS
//-----------------------------------------------------------------------------


void
ProbeViewer::createProbeCombo()
{
  probeCombo_ = new QComboBox(this);
  updateProbeCombo();
}



//-----------------------------------------------------------------------------
// PRIVATE SLOTS
//-----------------------------------------------------------------------------


void
ProbeViewer::changeProbe(int probeIndex)
{
  probe_ = (ProbeMeshItem*)(probeCombo_->itemData(probeIndex).value<void*>());
  if(probe_ != 0){
    probeCombo_->blockSignals(true);
    probeCombo_->setCurrentIndex(probeIndex);
    probeCombo_->blockSignals(false);
    setData(probe_->data());
  }else{
    setData();
  }
}


//-----------------------------------------------------------------------------
// PUBLIC SLOTS
//-----------------------------------------------------------------------------


void
ProbeViewer::updateProbeCombo()
{
  disconnect(probeCombo_,SIGNAL(currentIndexChanged(int)),
             this,SLOT(changeProbe(int)));
  probeCombo_->clear();
  QList<ProbeMeshItem*> meshProbes = mesh_->filteredItems<ProbeMeshItem>();
  if(meshProbes.count() > 0){
    QVariant data;
    int currentProbeNum = 0;
    foreach(ProbeMeshItem* probe, meshProbes){
      data.setValue((void*)probe);
      probeCombo_->addItem(probe->itemName(),data);
      if(probe_ == probe){
        currentProbeNum = probeCombo_->count()-1;
      }
    }
    probeCombo_->setEnabled(true);
    transformCombo_->setEnabled(true);
    changeProbe(currentProbeNum);
    connect(probeCombo_, SIGNAL(currentIndexChanged(int)),
            this, SLOT(changeProbe(int)));

  }else{
    probeCombo_->addItem(tr("No Probes"));
    probeCombo_->setEnabled(false);
    transformCombo_->setEnabled(false);
    changeProbe(0);

  }
}
