/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/
#include "animatorglmanager.h"
#include <QtOpenGL>
#include "openglscene.h"
#include "openglview.h"

//AnimatorGLManager::AnimatorGLManager(QObject *parent) :
//    QObject(parent),
//    animatorViewGL(QSharedPointer<OpenGLView>(new OpenGLView())),
//    meshPtr(QSharedPointer<Mesh>())
//{
//    renderThread = QSharedPointer<MeshRenderThread>();
//    animatorScene = QSharedPointer<OpenGLScene>(new OpenGLScene());
//    initialize();
//}

AnimatorGLManager::AnimatorGLManager(QSharedPointer<Mesh> mesh, QObject *parent) :
    QObject(parent),
    animatorViewGL(QSharedPointer<OpenGLView>(new OpenGLView())),
    meshPtr(mesh)
{
    qRegisterMetaType<MeshSize>("MeshSize");
    renderingThread = QSharedPointer<MeshRenderThread>(new MeshRenderThread(meshPtr->getMeshMutex(),this));
    animatorScene = QSharedPointer<OpenGLScene>(new OpenGLScene(renderingThread->getBufferMutex()));

    animatorViewGL->setScene(animatorScene.data());
    animatorViewGL->setViewportUpdateMode(QGraphicsView::FullViewportUpdate);
    //animatorViewGL->setViewport(new QGLWidget(QGLFormat(QGL::SampleBuffers)));


    QGLFormat f = QGLFormat::defaultFormat();
    f.setSampleBuffers(false);
    QGLFormat::setDefaultFormat(f);

//    QGLWidget glWidget;

//    if (!glWidget.format().sampleBuffers()) {
//	qDebug()<<"Sample Buffer Disabled";
//	f.setSampleBuffers(false);
//	QGLFormat::setDefaultFormat(f);
//    }else{
//	qDebug()<<"Sample Buffer Enabled";
//    }

    animatorViewGL->setViewport(new QGLWidget(QGLFormat::defaultFormat()));

    connectSignals();
}


AnimatorGLManager::~AnimatorGLManager(){
    animatorViewGL.clear();
    animatorScene.clear();
    meshPtr.clear();
    renderingThread.clear();
}


QSharedPointer<QWidget> AnimatorGLManager::getViewPtr()const{
    return animatorViewGL.dynamicCast<QWidget>();
}


void AnimatorGLManager::initialize(){
}

void AnimatorGLManager::connectSignals(){
    if(!meshPtr.isNull()){
        connect(meshPtr.data(),SIGNAL(voltageDataUpdated()),this,SLOT(meshDataChanged()));
    }
    connect(renderingThread.data(),SIGNAL(buffersGenerated(QVector<float>*,QVector<int>*,QVector<float>*,MeshSize)),
	       animatorScene.data(),SLOT(updateBuffers(QVector<float>*,QVector<int>*,QVector<float>*,MeshSize)));

}


// Slots


void AnimatorGLManager::setMesh(QSharedPointer<Mesh> newMesh){
    disconnect(this,SLOT(meshDataChanged()));
    disconnect(renderingThread.data(),SIGNAL(buffersGenerated(QVector<float>*,QVector<int>*,QVector<float>*,MeshSize)),
	       animatorScene.data(),SLOT(updateBuffers(QVector<float>*,QVector<int>*,QVector<float>*,MeshSize)));
    meshPtr = newMesh;
    renderingThread.clear();
    renderingThread = QSharedPointer<MeshRenderThread>(new MeshRenderThread(meshPtr->getMeshMutex(),this));
    connectSignals();
    meshDataChanged();
}

void AnimatorGLManager::meshDataChanged(){
    if(!meshPtr.isNull()){
	qDebug()<<"Rendering Thread Sleeping:"<<renderingThread->isSleeping()
		<<"Rendering Thread Running:"<<renderingThread->isRunning();
	if(renderingThread->isRunning()){
	    if(renderingThread->isSleeping()){
		renderingThread->generateBuffers(*meshPtr,animatorScene->getModelColor());
	    }
	}else{
	    renderingThread->generateBuffers(*meshPtr,animatorScene->getModelColor());
	}

    }
}

