/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "editorstatemachine.h"
#include "editor.h"
#include "editorview.h"
#include "editorview2d.h"
#include "editorview3d.h"
#include "meshviewgl.h"
#include "meshitems.h"
#include "pointmappergl.h"



//-----------------------------------------------------------------------------
// CONSTRUCTORS / DESTRUCTORS
//-----------------------------------------------------------------------------


EditorStateMachine::EditorStateMachine(Editor *editor)
  : QObject       ( 0 )
  , editor_       ( editor )
  , mesh_         ( editor->getMesh())
  , pointMapper_  ( editor_->getPointMapper() )
  , activeView_   ( 0 )

{
  createItemMenu();
}



//-----------------------------------------------------------------------------
// PUBLIC METHODS
//-----------------------------------------------------------------------------



QMenu*
EditorStateMachine::getItemActionMenu() const
{
  return itemMenu_;
}



//-----------------------------------------------------------------------------
// PROTECTED METHODS
//-----------------------------------------------------------------------------


bool
EditorStateMachine::eventFilter(QObject *object, QEvent *event)
{
//  MeshViewGL* meshView = qobject_cast<MeshViewGL*>(object);
//  if(meshView!=0){

//    EditorView* editorView = meshView->getEditorView();

//    if(editorView->is3DView()){
//      return process3DEvents(editorView, event);
//    }
//    else{
//      return process2DEvents(editorView, event);
//    }
//  }
  return false;
}




//-----------------------------------------------------------------------------
// PRIVATE METHODS
//-----------------------------------------------------------------------------


void EditorStateMachine::setState(State newState)
{
  currentState_ = newState;
  emit stateChanged(currentState_);
}



bool EditorStateMachine::isViewEnabled(EditorView2D *view)
{
  if(!support3D_){
    if(view->viewMask() == Point3D<float>(1,1,0)){
      return true;
    }
    return false;
  }
  return true;
}



void
EditorStateMachine::createItemMenu()
{
  itemMenu_ = new QMenu(tr("&Edit"),qobject_cast<QWidget*>(this));

  QAction* action;
  QIcon icon;

  icon = QIcon();
  icon.addFile(QString::fromUtf8(":/Images/Images/scaleNormalOn.png"), QSize(), QIcon::Normal, QIcon::On);
  action = new QAction(icon,tr("&Scale"),this);
  action->setObjectName(QString("actionScaleItems"));
  connect(action,SIGNAL(triggered()),this,SLOT(actionScaleItem()));
  itemMenu_->addAction(action);

//  icon = QIcon();
////  icon.addFile(QString::fromUtf8(":/Images/Images/moveNormalOn.png"), QSize(), QIcon::Normal, QIcon::On);
//  action = new QAction(icon,tr("&Move"),this);
//  action->setObjectName(QString("actionMoveItems"));
//  connect(action,SIGNAL(triggered()),this,SLOT(actionMoveItem()));
//  itemMenu_->addAction(action);

  icon = QIcon();
  icon.addFile(QString::fromUtf8(":/Images/Images/deleteNormalOn.png"), QSize(), QIcon::Normal, QIcon::On);
  action = new QAction(icon,tr("&Delete"),this);
  action->setShortcut(QKeySequence::Delete);
  action->setObjectName(QString("actionDeleteItems"));
  connect(action,SIGNAL(triggered()),this,SLOT(actionDeleteItem()));
  itemMenu_->addAction(action);

  connect(itemMenu_,SIGNAL(aboutToShow()),this,SLOT(itemMenuBeforeShow()));

}





void
EditorStateMachine::scale(const Size3D<float> size)
{
  mesh_.setItemsSize(size);
}



void
EditorStateMachine::adjustScale(const Size3D<float> deltaSize)
{
  mesh_.adjustItemsSize(deltaSize);
}



void EditorStateMachine::move(const Point3D<float> pos)
{
  mesh_.adjustItemsPos(pos);
}


void
EditorStateMachine::insertItem(const Point3D<float> pos,
                               const Size3D<float> size,
                               const Point3D<float> upVector)
{
  Size3D<float> newSize = Size3D<float>(size.toPoint3D()*upVector.mask());
  currentMeshItem_->setUp(upVector);
  currentMeshItem_->setPos(pos);
  currentMeshItem_->setSize(newSize);
  mesh_.insertItem(currentMeshItem_);
}




//-----------------------------------------------------------------------------
// PRIVATE SLOTS
//-----------------------------------------------------------------------------


void
EditorStateMachine::itemMenuBeforeShow()
{
  foreach(QAction* action, itemMenu_->actions()){
    if(action->objectName() == QString("actionScaleItems")){
      action->setEnabled(mesh_.selectedItems().count() == 1);
    }else{
      action->setEnabled(mesh_.selectedItems().count() > 0);
    }
  }
}


void
EditorStateMachine::actionDeleteItem()
{
  mesh_.deleteItems(mesh_.selectedItems());
}



void
EditorStateMachine::actionScaleItem()
{
  Point3D<float> bottomBackRight = getBottomBackLeft(mesh_.selectedItems());
  bottomBackRight.rx() = getTopFrontRight(mesh_.selectedItems()).cx();
  Point3D<float> topFrontLeft = getTopFrontRight(mesh_.selectedItems());
  topFrontLeft.rx() = getBottomBackLeft(mesh_.selectedItems()).cx();
  QPoint itemPos = pointMapper_.mapFromGLToView(bottomBackRight,activeView_);
  // Set the mouseTracker mousePress position to the topFrontLeft of the item
  // so that when we move the cursor to the bottomBackRight it will be set to the correct size
  pointMapper_.mouseTracker.setPressPosition(topFrontLeft,activeView_->viewMask());
  // Not sure why there is an offset but there is, could be because of the
  // split that this view is part of however, mapToGlobal should take care of
  // that.  Todo:  Figure out what is causing the offset, most likely in the
  // PointMapperGL class
  itemPos += QPoint(10,10);
  emit setCursorPos(activeView_->mapToGlobal(itemPos));
  setState(Scale);
  activeView_->setViewMouseTracking(true);
}




//-----------------------------------------------------------------------------
// PUBLIC SLOTS
//-----------------------------------------------------------------------------



void
EditorStateMachine::setCurrentItem(MeshItem *meshItem)
{
  currentMeshItem_ = meshItem;
  setState(Insert);
}



void
EditorStateMachine::itemInserted(const MeshItem *meshItem)
{
//  setState(Scale);
}



void
EditorStateMachine::keyPressed(Qt::Key key, Qt::KeyboardModifiers modifiers)
{

}



void
EditorStateMachine::mousePressed2D(const QPoint mousePos,
                                   Qt::MouseButtons buttons,
                                   Qt::KeyboardModifiers modifiers)
{
  activeView_ = qobject_cast<EditorView2D*>(QObject::sender());
  if(currentState_ == Insert){
    Point3D<float> antiMask = Point3D<float>(1,1,1)-activeView_->viewMask();
    Size3D<float> itemSize = pointMapper_.mouseTracker.getDeltaPress();
    insertItem(pointMapper_.mouseTracker.getPressPosition(),
               itemSize, antiMask);
  }
  if(currentState_ == Scale){
    activeView_->setViewMouseTracking(false);
    setState(Select);
  }
  if(currentState_ == Select){
    MeshItem* pickedItem = activeView_->pickItem(mousePos);
    if(pickedItem!=0){
      if(pickedItem->isSelected()){
        if(buttons.testFlag(Qt::RightButton)){
          itemMenu_->exec(activeView_->mapToGlobal(mousePos));
        }
      }else{
        if(!modifiers.testFlag(Qt::ControlModifier)){
          mesh_.blockSignals(true);
          mesh_.clearSelection();
          mesh_.blockSignals(false);
        }
          mesh_.selectItem(pickedItem);
      }
      if(buttons.testFlag(Qt::LeftButton)){
        setState(Move);
      }
      if(buttons.testFlag(Qt::MiddleButton)){
        setState(Scale);
      }
    }else{
      mesh_.clearSelection();
      if(buttons.testFlag(Qt::RightButton)){
        editor_->getEditorMenu()->exec(activeView_->mapToGlobal(mousePos));
      }
    }
  }

}



void
EditorStateMachine::mouseReleased2D(const QPoint mousePos,
                                    Qt::MouseButtons buttons,
                                    Qt::KeyboardModifiers modifiers)
{
  activeView_ = qobject_cast<EditorView2D*>(QObject::sender());
  setState(Select);
}



void
EditorStateMachine::mouseMoved2D(const QPoint mousePos,
                                 Qt::MouseButtons buttons,
                                 Qt::KeyboardModifiers modifiers)
{
  activeView_ = qobject_cast<EditorView2D*>(QObject::sender());

  if(currentState_ == Scale){
    if(buttons.testFlag(Qt::MiddleButton)){
      float depthDelta = dot(activeView_->vAxis(),
                             pointMapper_.mouseTracker.getMoveDelta());
      adjustScale((activeView_->vAxis().cross(activeView_->hAxis()))*depthDelta);
    }else{
      scale(pointMapper_.mouseTracker.getMovePressDelta());
    }
  }
  else if(currentState_ == Move){
    move(pointMapper_.mouseTracker.getMoveDelta()*activeView_->viewMask());
  }
  else if(currentState_ == Insert){
    scale(pointMapper_.mouseTracker.getMovePressDelta());
  }
}



void
EditorStateMachine::mousePressed3D(const QPoint mousePos,
                                   Qt::MouseButtons buttons,
                                   Qt::KeyboardModifiers modifiers)
{
  EditorView3D* senderView = qobject_cast<EditorView3D*>(QObject::sender());
  MeshItem* pickedItem = senderView->pickItem(mousePos);
  if(pickedItem!=0){
    if(modifiers.testFlag(Qt::ControlModifier)){
      if(pickedItem->isSelected()){
        mesh_.deselectItem(pickedItem);
      }else{
        mesh_.selectItem(pickedItem);
      }
    }else{
      if(!pickedItem->isSelected()){
        mesh_.blockSignals(true);
        mesh_.clearSelection();
        mesh_.blockSignals(false);
        mesh_.selectItem(pickedItem);
      }
    }
  }
  else{
    mesh_.clearSelection();
  }

}



void
EditorStateMachine::changeItemProperty(QString propertyName, QVariant value)
{
  qDebug()<<"Changed selected item properties";
  foreach(MeshItem* item, mesh_.selectedItems()){
    item->setPropertyValue(propertyName,value);
  }
}
