/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef EDITORSTATEMACHINE_H
#define EDITORSTATEMACHINE_H

#include <QObject>
#include <QtGui>
#include "point3d.h"
#include "size3d.h"
class MeshItem;
class Editor;
class EditorView;
class EditorView2D;
class PointMapperGL;
class Mesh;
class MeshEngine;

class EditorStateMachine : public QObject
{
  Q_OBJECT
public:
  enum State{Select,Move,Insert,Scale};
private:
  State               currentState_;
  MeshItem*           currentMeshItem_;
  Editor*             editor_;
  Mesh&               mesh_;
  PointMapperGL&      pointMapper_;
  bool                support3D_;
  QMenu*              itemMenu_;
  EditorView2D*       activeView_;

public:
  explicit EditorStateMachine(Editor *editor = 0);
  QMenu* getItemActionMenu() const;

protected:
  bool eventFilter(QObject *object, QEvent *event);


private:

//  bool process3DEvents(EditorView *editorView, QEvent *event);
//  bool process2DEvents(EditorView *editorView, QEvent *event);
  void createItemMenu();
  void scale(const Size3D<float> size);
  void adjustScale(const Size3D<float> deltaSize);
  void insertItem(const Point3D<float> pos,
                  const Size3D<float> size,
                  const Point3D<float> upVector);
  void move(const Point3D<float> pos);

  void setState(State newState);
  bool isViewEnabled(EditorView2D* view);

signals:
  void moveItems( const Point3D<float> pos);
  void scaleItems( const Size3D<float> size);
  void itemReadyToInsert( const MeshItem* meshItem);
  void stateChanged(State newState);
  void setCursorPos(QPoint pos);

private slots:
  void itemMenuBeforeShow();
  void actionDeleteItem();
  void actionScaleItem();

public slots:
  void setCurrentItem(MeshItem* meshItem);
  void itemInserted(const MeshItem* meshItem);

  void keyPressed(Qt::Key key, Qt::KeyboardModifiers modifiers);
  void mousePressed2D(const QPoint mousePos,
                      Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers);
  void mouseReleased2D(const QPoint mousePos,
                       Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers);
  void mouseMoved2D(const QPoint mousePos,
                    Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers);
  void mousePressed3D(const QPoint mousePos,
                      Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers);

  void changeItemProperty(QString propertyName, QVariant value);

};


#endif // EDITORSTATEMACHINE_H
