/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "editorview.h"
#include "editor.h"
#include "meshviewgl.h"
#include "pointmappergl.h"
#include "meshitems.h"

EditorView::EditorView(Editor *parent) :
    QWidget       (0),
    editor_       (parent),
    pointMapper_  (editor_->getPointMapper()),
    mainLayout_   (new QGridLayout(this)),
    scrollH_      (new QScrollBar(Qt::Horizontal)),
    scrollV_      (new QScrollBar(Qt::Vertical)),
    viewGL_       (new MeshViewGL(pointMapper_,this))
{
  viewGL_->setCameraAt(Point3D<GLfloat>(0,0,0));
  glLayout_ = new QHBoxLayout;
  glLayout_->addWidget(viewGL_);
  spacerItem = new QSpacerItem(1,1,QSizePolicy::Minimum,QSizePolicy::Expanding);
  glLayout_->addSpacerItem(spacerItem);

  mainLayout_->addLayout(glLayout_,0,0);
  //setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
  scrollH_->setRange(0,pointMapper_.mapGLToPixel(pointMapper_.worldSize()/2));
  scrollV_->setRange(0,pointMapper_.mapGLToPixel(pointMapper_.worldSize()/2));
  centerHScrollBar();
  centerVScrollBar();

//  // THE FOLLOWING SHOULD BE REPLACED BY SUBCLASSING THIS EDITORVIEW CLASS
//  if(!is3DView_){
////    viewGL_->installEventFilter((QObject*)(editor_->getStateMachine()));
////    // Initialize scroll bars
////    scrollH_->setRange(0,pointMapper_.mapGLToPixel(pointMapper_.worldSize()/2));
////    scrollV_->setRange(0,pointMapper_.mapGLToPixel(pointMapper_.worldSize()/2));
////    centerHScrollBar();
////    centerVScrollBar();

////    Point3D<float> crossPoint = cross(vAxis_,hAxis_);
////    // If it is the top view
////    if(qAbs(crossPoint.cy()) == 1){
////      rotateAroundX(-90);
////      vAxisInverted_ = true;
////    }
////    // if it is the side view
////    if(qAbs(crossPoint.cx()) == 1){
////      rotateAroundY(90);
////      hAxisInverted_ = true;
////    }
////    // if it is the front view
////    if(qAbs(crossPoint.cz()) == 1){
////      vAxisInverted_ = true;
////    }


//  connect(scrollH_,SIGNAL(valueChanged(int)),this,SIGNAL(hScrollChanged(int)));
//  connect(scrollV_,SIGNAL(valueChanged(int)),this,SIGNAL(vScrollChanged(int)));

////    connect(viewGL_,SIGNAL(viewPortResized(QSize)),
////            this,SLOT(updateScrollRange(QSize)));

////    connect(&pointMapper_,SIGNAL(sceneDepthsChanged(Point3D<float>))
////            ,this ,SLOT(sceneDepthsChanged(Point3D<float>)));

//  }else{
////    scrollH_->setDisabled(true);
////    scrollV_->setDisabled(true);
//  }

  connect(viewGL_,SIGNAL(viewPortResized(QSize)),
          this,SLOT(updateScrollRange(QSize)));

  mainLayout_->addWidget(scrollH_,1,0);
  mainLayout_->addWidget(scrollV_,0,1);

  setLayout(mainLayout_);


}




//-----------------------------------------------------------------------------
//  PUBLIC METHODS
//-----------------------------------------------------------------------------


void
EditorView::setCameraDistance(const GLfloat distance)
{
  viewGL_->setCameraDistance(distance);
}



void
EditorView::setCameraPos(const Point3D<GLfloat> cameraPos,
                         const Point3D<float> changeVector)
{
  viewGL_->setCameraPos(cameraPos,changeVector);
}



void
EditorView::moveCamera(const Point3D<GLfloat> deltaPos)
{
  viewGL_->moveCamera(deltaPos);
}



void
EditorView::setRotation(const Point3D<GLfloat> newRotation)
{
  viewGL_->setRotation(newRotation);
}



void
EditorView::rotateAroundX(GLfloat angle)
{
  viewGL_->rotateAroundX(angle);
}



void
EditorView::rotateAroundY(GLfloat angle)
{
  viewGL_->rotateAroundY(angle);
}



void
EditorView::rotateAroundZ(GLfloat angle)
{
  viewGL_->rotateAroundZ(angle);
}



MeshItem*
EditorView::pickItem(const QPoint& pos)
{
  return viewGL_->pickItem(pos);
}


const QSize
EditorView::viewPortSize() const
{
  return viewGL_->viewPortSize();
}


GLfloat
EditorView::getCameraDistance() const
{
  return viewGL_->getCameraDistance();
}



Point3D<GLfloat>
EditorView::getCameraPos() const
{
  return viewGL_->getCameraPos();
}



Point3D<GLfloat>
EditorView::getRotation() const
{
  return viewGL_->getRotation();
}



void EditorView::setupGLView(QSize viewSize)
{
  // Need to have this method defined in order to avoid
  // runtime Assertion due to the QGLWidget calling this
  // function on object construction.
}



//-----------------------------------------------------------------------------
//  PROTECTED METHODS
//-----------------------------------------------------------------------------


void EditorView::keyPressEvent(QKeyEvent *event)
{

}




//-----------------------------------------------------------------------------
//  PRIVATE METHODS
//-----------------------------------------------------------------------------

void
EditorView::updateScrollRange(QSize viewPortSize)
{
  int worldSizePixels = pointMapper_.mapGLToPixel(pointMapper_.worldSize()/2.0);
  int pageStepX = viewPortSize.width();
  int pageStepY = viewPortSize.height();

  scrollH_->setMaximum(worldSizePixels - pageStepX/2);
  scrollH_->setMinimum(pageStepX/2);
  scrollH_->setPageStep(pageStepX);

  scrollV_->setMaximum(worldSizePixels - pageStepY/2);
  scrollV_->setMinimum(pageStepY/2);
  scrollV_->setPageStep(pageStepY);

  sceneDepthsChanged(pointMapper_.sceneDepths());
}




//-----------------------------------------------------------------------------
//  PUBLIC SLOTS
//-----------------------------------------------------------------------------


void
EditorView::sceneDepthsChanged(Point3D<float> depths)
{

}



void
EditorView::setHScrollValue(int value)
{
  scrollH_->setValue(value);
}



void
EditorView::setVScrollValue(int value)
{
  scrollV_->setValue(value);
}



void
EditorView::centerHScrollBar()
{
  int meanValue = pointMapper_.mapGLToPixel(0);
  scrollH_->setValue(meanValue);
}



void
EditorView::centerVScrollBar()
{
  int meanValue = pointMapper_.mapGLToPixel(0);
  scrollV_->setValue(meanValue);
}



void
EditorView::sendScrollHEvent(QEvent *event)
{
  QCoreApplication::sendEvent( static_cast<QObject*>(scrollH_), event);
}



void
EditorView::sendScrollVEvent(QEvent *event)
{
  QCoreApplication::sendEvent( static_cast<QObject*>(scrollV_), event);
}





