/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#include "mousetracker.h"

MouseTracker::MouseTracker() :
    QObject(0),
    showPressPositions_ ( false )
{
}



//MouseTracker::MouseTracker(MouseTracker& other) :
//    movePosition_     ( other.movePosition_ ),
//    releasePosition_  ( other.releasePosition_ ),
//    showPressPositions_ ( other.showPressPositions_ )
//{
//  pressPosition_[0] = other.pressPosition_[0];
//  pressPosition_[1] = other.pressPosition_[1];
//  movePosition_[0]    = other.movePosition_[0];
//  movePosition_[1]    = other.movePosition_[1];
//}



MouseTracker&
MouseTracker::operator =(MouseTracker& other)
{
  if(&other != this){
    pressPosition_[0]   = other.pressPosition_[0];
    pressPosition_[1]   = other.pressPosition_[1];
    movePosition_[0]    = other.movePosition_[0];
    movePosition_[1]    = other.movePosition_[1];
    releasePosition_    = other.releasePosition_;
    showPressPositions_ = other.showPressPositions_;
  }
  return *this;
}



//-----------------------------------------------------------------------------
// PUBLIC METHODS
//-----------------------------------------------------------------------------



void
MouseTracker::showPressPositions(const bool show)
{
  showPressPositions_ = show;
  pressPositionChanged(pressPosition_[0], pressPosition_[1]);
}



void
MouseTracker::setPressPosition(const Point3D<float> pressPosition,
                               Point3D<float> viewMask)
{
  // Update the 'first' press position with the previous 'last' position
  pressPosition_[1].rx() = (viewMask.cx() == 1)
                           ? pressPosition_[0].cx() : pressPosition_[1].cx();
  pressPosition_[1].ry() = (viewMask.cy() == 1)
                           ? pressPosition_[0].cy() : pressPosition_[1].cy();
  pressPosition_[1].rz() = (viewMask.cz() == 1)
                           ? pressPosition_[0].cz() : pressPosition_[1].cz();
  // Update the 'last' press position with the current press position
  pressPosition_[0].rx() = (viewMask.cx() == 1)
                           ? pressPosition.cx() : pressPosition_[0].cx();
  pressPosition_[0].ry() = (viewMask.cy() == 1)
                           ? pressPosition.cy() : pressPosition_[0].cy();
  pressPosition_[0].rz() = (viewMask.cz() == 1)
                           ? pressPosition.cz() : pressPosition_[0].cz();

  emit pressPositionChanged(pressPosition_[0],pressPosition_[1]);
}



void
MouseTracker::setMovePosition(const Point3D<float> movePosition,
                              Point3D<float> viewMask)
{
  movePosition_[1].rx() = (viewMask.cx() == 1)
                           ? movePosition_[0].cx() : movePosition_[1].cx();
  movePosition_[1].ry() = (viewMask.cy() == 1)
                           ? movePosition_[0].cy() : movePosition_[1].cy();
  movePosition_[1].rz() = (viewMask.cz() == 1)
                           ? movePosition_[0].cz() : movePosition_[1].cz();
  // Update the 'last' press position with the current press position
  movePosition_[0].rx() = (viewMask.cx() == 1)
                           ? movePosition.cx() : movePosition_[0].cx();
  movePosition_[0].ry() = (viewMask.cy() == 1)
                           ? movePosition.cy() : movePosition_[0].cy();
  movePosition_[0].rz() = (viewMask.cz() == 1)
                           ? movePosition.cz() : movePosition_[0].cz();

  emit movePositionChanged(movePosition_[0],movePosition_[1]);
}



void
MouseTracker::setReleasePosition(const Point3D<float> releasePosition,
                                 Point3D<float> viewMask)
{
  releasePosition_.rx() = (viewMask.cx() == 1)
                           ? releasePosition.cx() : releasePosition_.cx();
  releasePosition_.ry() = (viewMask.cy() == 1)
                           ? releasePosition.cy() : releasePosition_.cy();
  releasePosition_.rz() = (viewMask.cz() == 1)
                           ? releasePosition.cz() : releasePosition_.cz();

  emit releasePositionChanged(releasePosition_);
}



const bool
MouseTracker::pressPositionsVisible() const
{
  return showPressPositions_;
}



const Point3D<float>&
MouseTracker::getPressPosition() const
{
  return pressPosition_[0];
}



const Point3D<float>&
MouseTracker::getLastPressPosition() const
{
  return pressPosition_[1];
}



const Point3D<float>&
MouseTracker::getMovePosition() const
{
  return movePosition_[0];
}



const Point3D<float>&
MouseTracker::getLastMovePosition() const
{
  return movePosition_[1];
}



const Point3D<float>&
MouseTracker::getReleasePosition() const
{
  return releasePosition_;
}



// Difference between the two mouse press positions
const Point3D<float>
MouseTracker::getDeltaPress() const
{
  return pressPosition_[0]-pressPosition_[1];
}



// Difference between the last press position and the current move position
const Point3D<float>
MouseTracker::getMovePressDelta() const
{
  return movePosition_[0] - pressPosition_[0];
}



const Point3D<float>
MouseTracker::getMoveDelta() const
{
  return movePosition_[0] - movePosition_[1];
}



// Difference between the last press position and the release position
const Point3D<float>
MouseTracker::getReleasePressDelta() const
{
  return releasePosition_ - pressPosition_[0];
}
