/*******************************************************************************
**  FILE:         pointmappergl.h
**  DESCRIPTION:  This file describes the PointMapperGL object which is
**                Responsible for mapping between view coordinates (pixels)
**                and OpenGL scene coordinates.
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef POINTMAPPERGL_H
#define POINTMAPPERGL_H

#include <QtGui>
#include <QWidget>
#include "point3d.h"
#include "size3d.h"
#include "mousetracker.h"
class EditorView2D;


//class PointMapperGLDialog : public QDialog
//{
//  Q_OBJECT
//private:
////  QLineEdit worldSizeEdit_;
////  QLineEdit drawingResolution_;
////  QLineEdit minorGridSpacingEdit_;
////  QLineEdit majorGridSpacingEdit_;
////  QCheckBox gridVisibleBox_;
////  QCheckBox snapToGridBox_;



//public:


//};



class PointMapperGL : public QWidget
{
    Q_OBJECT
private:
  float           worldSize_;
  float           drawingResolution_;
  float           zoomFactor_;
  float           glSceneScale_;
  Point3D<float>  sceneDepths_;
  int             minorGridSpacing_;
  int             majorGridSpacing_;
  bool            gridVisibleFlag_;
  static bool     snapToGridFlag_;

public:
  MouseTracker    mouseTracker;

public:
  explicit PointMapperGL(float worldSize,
                         float drawingResolution,
                         float zoomFactor = 1.0);

  float convertPixelToGL(const int pixel,
                         const bool snap = snapToGridFlag_) const;
  float mapPixelToGL(const int pixel,
                     const bool snap = snapToGridFlag_ ) const;
  Point3D<float> mapFromViewToGL(QPoint viewPoint, EditorView2D* view,
                                 const bool snap = snapToGridFlag_) const;

  int convertGLToPixel(const float glCoord) const;
  int mapGLToPixel(const float glCoord) const;
  QPoint mapFromGLToView(const Point3D<float> glPoint,
                         EditorView2D* view) const;

  const float worldSize() const;
  const float drawingResolution() const;
  const float zoomFactor() const;
  const float glSceneScale() const;
  const Point3D<float> sceneDepths() const;
  const int minorGridSpacing() const;
  const int majorGridSpacing() const;
  const bool gridVisible() const;
  const bool snapToGrid() const;

signals:
  void worldSizeChanged(const float newWorldSize);
  void drawingResolutionChanged(const float newResolution);
  void zoomFactorChanged(const float newZoomFactor);
  void glSceneScaleChanged(const float newGLSceneScale);
  void sceneDepthsChanged(const Point3D<float> newDepths);
  void sceneDepthXChanged(const float newDepth);
  void sceneDepthYChanged(const float newDepth);
  void sceneDepthZChanged(const float newDepth);
  void minorGridSpacingChanged(const int newSpacing);
  void majorGridSpacingChanged(const int newSpacing);
  void gridVisibleChanged(const bool visible);
  void snapToGridChanged(const bool snap);

public slots:
  void setWorldSize(const float worldSize);
  void setDrawingResolution(const float drawingResolution);
  void setZoomFactor(const float zoomFactor);
  void setGLSceneScale(const float glSceneScale);
  void setSceneDepths(const Point3D<float> newDepths);
  void setSceneDepthX(const float newDepth);
  void setSceneDepthY(const float newDepth);
  void setSceneDepthZ(const float newDepth);
  void setMinorGridSpacing(const int newSpacing);
  void setMajorGridSpacing(const int newSpacing);
  void setGridVisible(const bool visible);
  void setSnapToGrid(const bool snap);

};

#endif // POINTMAPPERGL_H
