/*******************************************************************************
**  FILE:	    meshsimulationthread.h
**  DESCRIPTION:    Thread used for running the simulation routines in a different
**		    thread (derrived from QThread class).  The simulation thread
**		    makes the appropriate calls to the MeshEngine class which
**		    provides the interface for simulating a Mesh.
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MESHSIMULATIONTHREAD_H
#define MESHSIMULATIONTHREAD_H

#include <QtCore>
#include <QMutex>
#include <QSize>
#include <QThread>
#include <QWaitCondition>

class MeshEngine;

class MeshSimulationThread : public QThread
{
    Q_OBJECT
private:
    QWaitCondition condition;
    QMutex simMutex;
    bool restart;
    bool abort;
    bool pause;
    MeshEngine * meshEngine;

    unsigned long delayFactor;
    unsigned int totalIterations;
    unsigned int updateInterval;
    unsigned int currentIteration;
    QTime        elapsedTime;


public:
    explicit MeshSimulationThread(QObject *parent = 0);
    ~MeshSimulationThread();

    void simulate(MeshEngine* engine,unsigned int iterations,unsigned int initUpdateInterval,unsigned long simDelayFactor = 50);
    bool isPaused()
    {
      return pause;
    }

protected:
    void run();

signals:
    void simulationComplete();
    void iterationCompleted(int iteration);

public slots:
    void stopSimulation();
    void pauseSimulation();
    void resumeSimulation();


};

#endif // MESHSIMULATIONTHREAD_H
