/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MESHSIMULATOR_H
#define MESHSIMULATOR_H

#include <QWidget>
#include <QtGui>
#include "UI/MeshEditors/editor.h"

class Mesh;
class MeshEngine;
class MeshSimulationThread;

class MeshSimulator : public QWidget
{
    Q_OBJECT

private:
  Mesh*                     mesh_;
  MeshSimulationThread*     simulationThread_;
  MeshEngine*               meshEngine_;
  bool                      meshModifiedFlag_;
  int                       totalIterations_;
  int                       progressPercent_;
  QTimer*                   statUpdateTimer_;


  // USER INTERFACE ELEMENTS

  QToolBar*                 toolBarSimulation;

  QPushButton*              buttonCreateMesh;
  QPushButton*              buttonRunSimulation;
  QPushButton*              buttonResetSimulation;
  QPushButton*              buttonClearSimulation;
  QLabel*                   labelSimProgress;
  QLabel*                   labelTotalIterations;
  QLabel*                   labelUpdateInterval;
  QLabel*                   labelDelayFactor;
  QProgressBar*             progressBarSimulation;
  QSlider*                  sliderTotalIterations;
  QSlider*                  sliderUpdateInterval;
  QSlider*                  sliderDelayFactor;
  QSpinBox*                 spinTotalIterations;
  QSpinBox*                 spinUpdateInterval;
  QSpinBox*                 spinDelayFactor;

  QMenu*                    meshEngineMenu;




public:
  explicit MeshSimulator(Mesh* mesh,
                         QWidget *parent = 0);
  ~MeshSimulator();

  QToolBar* getSimulationToolBar() const;
  QMenu* getMeshEngineMenu() const;
  MeshEngine* getMeshEngine() const;

private:
  void readSettings();
  void writeSettings();
  void createSimulationToolBar();
  void createMeshEngineMenu();
  void setMeshModifiedFlag(const bool flag);
  void setTotalIterationsEnabled(bool enabled);
  void setUpdateIntervalEnabled(bool enabled);
  void setDelayFactorEnabled(bool enabled);

protected:
  void changeEvent(QEvent *event);

signals:
  void meshEngineChanged(MeshEngine* meshEngine);
  void updateDisplays(MeshEngine* meshEngine);
  void updateUI();
  void iterationChanged(int iteration);

private slots:
  void meshEngineSelectionChanged();
  void setMeshEngine(MeshEngine* meshEngine);
  void meshEngineDataUpdated();
  void uiUpdateRequested();
  void iterationFinished(int iteration);
  void simulationComplete();
  void updateStats();

public slots:
  void editorModeChanged(Editor::EditorMode mode);
  void startSimulation();
  void pauseSimulation();
  void resetMesh();
  void clearSimulation();
  void createMesh();
  void changeTotalIterations(int totalIterations);
  void changeUpdateInterval(int updateInterval);
  void changeDelayFactor(int delayFactor);
  void setProgress(QString text,int value);
  void meshModified(Mesh * mesh);



};

#endif // MESHSIMULATOR_H
