/*******************************************************************************
**  FILE:
**  DESCRIPTION:
**
**
**
**  AUTHOR:	    Colter McQuay
**  ORGANIZATION:   University Of Victoria
**  SUPERVISOR:	    Dr. Poman So
**
*******************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QtGui>


QT_BEGIN_NAMESPACE
class QAction;
class QMenu;
class QMdiArea;
class QMdiSubWindow;
class QSignalMapper;
class QGLWidget;
class QGraphicsSceneMouseEvent;
class QPointF;
class QColor;
class QtOpenGL;
QT_END_NAMESPACE

class Editor;
class Animator;
class Mesh;
class MeshSimulator;
class MeshEngine;
class DataViewer;


class MainWindow : public QMainWindow {
    Q_OBJECT

//--------------------- MEMBERS ------------------------
private:
  QSharedPointer<QCursor> cursor;
  Mesh*                   mesh;
  Editor*                 meshEditor;
  MeshSimulator*          meshSimulator;
  MeshEngine*             meshEngine;


  //------------------------------------
  //   USER INTERFACE COMPONENTS
  //------------------------------------

  QLayout*        mainLayout;
  QStatusBar*     statusBar;
  QMenuBar*       menuBar;

  QMenu*          menuFile;
  QMenu*          menuWindow;
  QMenu*          menuOpenWindows;

  QMenu*          menuView;
  QMenu*          menuTools;
  QMenu*          menuHelp;


  QMdiArea*       mdiArea;
  QSignalMapper*  windowMapper;

  QMdiSubWindow*  editorWindow;


//--------------------- METHODS ------------------------
public:
  MainWindow(QWidget *parent = 0);
  ~MainWindow();

protected:
  void changeEvent(QEvent *e);
  void closeEvent(QCloseEvent *event);

private:
  void preInitialize();
  void connectSignals();
  void initialize();

  void readSettings();
  void writeSettings();

  //    void createMeshItemList();
  //    void createPropertyList(QTreeWidgetItem* parentTreeItem,MeshItem* item);
  //    void createShapeList(QTreeWidgetItem* parentTreeItem,QList<MeshItem*> itemList);
  //    QMap<QString,QVariant> getListItemProperties(QTreeWidgetItem* selectedItem);
  //    QVariant getListItemPropertyValue(QTreeWidgetItem* propertyListItem);
  void createFileMenu();
  void createWindowMenu();
  void createMenuBar();
  void updateWindowMenu();

private slots:
  void beforeShowFileMenu();
  void beforeShowWindowMenu();
  void createNewProbeViewer();
  void createNewAnimator();


public slots:
  void mdiWindowActivated(QMdiSubWindow* subWindow);

  // MISC SLOTS
  void setCursorShape(Qt::CursorShape shape);
  void setCursorPos(QPoint pos);
  void setStatus(const QString status, const int timeOut = 0);
  void iterationChanged(int iteration);

  // FILE SLOTS
  // UI -> MainWindow
  void fileNew();
  void fileOpen();
  void fileSave();
  void fileSaveAs();

  void setActiveSubWindow(QWidget* window);

};

#endif // MAINWINDOW_H
